/*  samples/ccstatud_sample.p  Update CC transaction status  SAMPLE PROGRAM
            DY Tri-8, 03/07/2002                                  
*/

/**************************************************************************
$Id: ccstatud_sample.p,v 1.3 2004/07/02 15:31:15 dan Exp $

  Author: Dan Yost <dan@tri8.com>
    Date: 03/07/2002

   USAGE: ccstatud ( input-output temp-table_trExec )

  OUTPUT: trExec record matching the updated cc-trans record 

  PREFACE:
    This program is provided as a sample of how one might use OneStep Charge
    to interface with a Progress database.  This program relies on a table
    called "cc-trans" which is assumed to exist in your database as the 
    table for permanent credit card transaction storage.  

  DESCRIPTION:
     This program updates cc-trans records with the latest data for those
     records by querying the OSC software.  This query is done via the 
     standard OSC API, letting the OSC controller actually do the talking to 
     the OSC software.  For each trExec record, the cc-trans record represented
     by that trExec record (using the transNo) is *updated* according to the 
     results of the OSC controller call.
     
**************************************************************************/

 /************************************************************************
 * COPYRIGHT Tri-8, Inc. This work is unpublished.  ALL RIGHTS RESERVED. * 
 ************************************************************************/

/*************************** DEFINITIONS *********************************/
{tri8osc/oscapi}

def input-output parameter table for trExec.

def var ccp-runpgm as char init "tri8osc/oscengin.p" no-undo.


/*************************** MAIN BLOCK **********************************/

if not can-find(first trExec) then return.

/* update all trExec records -- we force everything to CCA_STATUS 
// regardless of what was passed in, lest we accidentally send new 
// transactions to the CCP software */
for each trExec exclusive-lock:
    assign trExec.action = CCA_STATUS 
           trExec.timeout = if trExec.timeout = 0 
                            then OSCConfig.CCTimeout
                            else trExec.timeout
    no-error.

    if error-status:error
    then do:
        /* if we can't assign trExec.action then we're out of here, rather 
	// than risking sending live (new) transactions */

	/* send alert, log, etc. here */

        return.
    end.
end. /* for each trExec */


/* dispatch the CCP controller, which handles the actual processing,
// according to the standard OSC API */
  
/* NOTE: in some cases a run statement can cause a "stop" condition without
// causing an error condition, so we use a little more cumbersome structure
// in order to catch such things */
DSPBLOCK:
do on stop undo DSPBLOCK, retry DSPBLOCK
   on endkey undo DSPBLOCK, retry DSPBLOCK
   on error undo DSPBLOCK, retry DSPBLOCK transaction:
       
    if retry 
    then do:
      
	/* log/alert here */

        leave DSPBLOCK.
    end.
        
    /* dispatch the CCP controller */
    run value(ccp-runpgm) ( input-output table trExec ) no-error.
    
    if error-status:error then undo DSPBLOCK, retry DSPBLOCK.
        
    /* record all of the results */
    RESLOOP:
    for each trExec no-lock:
        find cc-trans exclusive-lock
          where cc-trans.trans-no = trExec.transNo
          no-wait no-error.
              
        if error-status:error
        then do:
            /* add any logging/recording here */
               
            next RESLOOP.
        end.

        /* also skip the recording of the results if we had a CCS_SUBERR
        // or CCS_SYSERR and the results are still a CCS_SUBERR/SYSERR */
        if ( cc-trans.authStatus = CCS_SUBERR
          or cc-trans.authStatus = CCS_SYSERR )
         and 
           ( trExec.authStatus = CCS_SUBERR
          or trExec.authStatus = CCS_SYSERR )
        then next RESLOOP.
         
        /* assign fields (buffer-copy if possible) */
        buffer-copy trExec except trExec.cardNo to cc-trans no-error.

        if error-status:error
        then do:
            /* add any logging/recording here */
            next RESLOOP.
        end. /* if error-status:error in copying trExec */

    end.  /* RESLOOP */
end.  /* DSPBLOCK */
    
